#include <floatStructs.h>

namespace sven {
using namespace std;

ostream& operator<<(ostream &os, float2 bob) {
	return os << bob[0] << " " << bob[1];
}

ostream& operator<<(ostream &os, float3 bob) {
	return os << bob[0] << " " << bob[1] << " "<< bob[2];
}

ostream& operator<<(ostream &os, float16 foo) {
	return os << "\n" << 
		"  " << foo[0] << " " << foo[1] << " " << foo[2] << " " << foo[3] << "\n" << 
		"  " << foo[4] << " " << foo[5] << " " << foo[6] << " " << foo[7] << "\n" << 
		"  " << foo[8] << " " << foo[9] << " " << foo[10] << " " << foo[11] << "\n" <<
		"  " << foo[12] << " " << foo[13] << " " << foo[14] << " " << foo[15];
}

void inMatrix(float * m, ifstream &in) {
	in >> m[0] >> m[1] >> m[2] >> m[3] >>
		  m[4] >> m[5] >> m[6] >> m[7] >>
		  m[8] >> m[9] >> m[10] >> m[11] >>
		  m[12] >> m[13] >> m[14] >> m[15];
}

void inFloat16(float16 &m, ifstream &in) {
	in >> m[0] >> m[1] >> m[2] >> m[3] >>
		  m[4] >> m[5] >> m[6] >> m[7] >>
		  m[8] >> m[9] >> m[10] >> m[11] >>
		  m[12] >> m[13] >> m[14] >> m[15];
}

void inFloat3(float3 &vec, ifstream &in) {
	in >> vec[0] >> vec[1] >> vec[2];
}

void inFloat2(float2 &point, ifstream &in) {
	in >> point[0] >> point[1];
}

///////////////////////////
//Mathy
//Note that float16's are assumed to be collum major (a la openGL) matrices.

float16 operator *(float16 a, float16 b) {
	float16 rval;
	float sum;
	for(int r=0;r<4;r++) {
		for(int c=0;c<4; c++) {
			sum=0;
			for(int q=0;q<4;q++) sum+=a[4*r+q]*b[q*4+c];
			rval[r*4+c]=sum;
		}
	}
	return rval;
}


float3 operator *(float16 a, float3 b) {
	float3 r(a[0]*b[0]+a[4]*b[1]+a[8]*b[2]+a[12],
			a[1]*b[0]+a[5]*b[1]+a[9]*b[2]+a[13],
			a[2]*b[0]+a[6]*b[1]+a[10]*b[2]+a[14]);
	return r;
}

float3 operator +(const float3 a, const float3 b) {
	float3 r(a[0]+b[0],
			a[1]+b[1],
			a[2]+b[2]);
	return r;
}

float3 operator -(const float3 a, const float3 b) {
	float3 r(a[0]-b[0],
			a[1]-b[1],
			a[2]-b[2]);
	return r;
}

float3 operator *(float c, float3 m) {
	float3 r(c*m[0],
			c*m[1],
			c*m[2]);
	return r;
}

float3 operator *(float3 m, float c) {
	float3 r(c*m[0],
			c*m[1],
			c*m[2]);
	return r;
}

float3 normalize(float3 p) {
	if(p[0]==p[1] && p[1]==p[2] && p[2]==0) return p;
	float mag2=p[0]*p[0]+p[1]*p[1]+p[2]*p[2];
	if(mag2==1) return p;
	float mag=(float)sqrt(mag2);
	p[0]/=mag;
	p[1]/=mag;
	p[2]/=mag;
	return p;
}

//returns a X b
float3 cross(float3 a, float3 b) {
	float3 r(
		a[1]*b[2]-b[1]*a[2],
		b[0]*a[2]-a[0]*b[2],
		a[0]*b[1]-b[0]*a[1]);
	return r;
}

//returns a . b
float dot(float3 a, float3 b) {
	return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}

float mag2(float3 m) {
	return m[0]*m[0]+m[1]*m[1]+m[2]*m[2];
}

float mag(float3 m) {
	return sqrtf(m[0]*m[0]+m[1]*m[1]+m[2]*m[2]);
}

}
